import datetime

import pandas as pd
import requests

def r_package_downloads(start, end):
    if start is None:
        start = '2020-01-01'
    resp = requests.get(f'https://cranlogs.r-pkg.org/downloads/total/{start}:{end}/nixtlar')
    return resp.json()[0]['downloads']

def fmt_with_change(lib, curr, prev):
    change = curr / prev - 1
    if change < 0:
        arr = '&#8595;'
        quant = 'less'
    else:
        arr = '&#8593;'
        quant = 'more'
    return f'{curr:,} {lib} ({arr} {change:.2%} {quant} than last week)'

this_week_end = datetime.date.today() - datetime.timedelta(days=1)
last_week_end = this_week_end - datetime.timedelta(days=7)
libs = ['statsforecast', 'nixtla', 'hierarchicalforecast', 'mlforecast', 'neuralforecast', 'tsfeatures', 'nixtlats']
res = {}
for lib in libs:
    resp = requests.get(
        url=f'https://api.pepy.tech/api/v2/projects/{lib}',
        headers={'X-Api-Key': 'eka05a/uP4IRXUyAqlFvME+vht31vi1D'},
    ).json()
    this_week_downloads = 0
    last_week_downloads = 0
    for i in range(7):
        this_week_downloads += sum(resp['downloads'][str(this_week_end - datetime.timedelta(days=i))].values())
        last_week_downloads += sum(resp['downloads'][str(last_week_end - datetime.timedelta(days=i))].values())
    res[lib] = {
        'total': resp['total_downloads'],
        'this_week': this_week_downloads,
        'last_week': last_week_downloads,
    }


# R
this_week_start = this_week_end - datetime.timedelta(days=6)
last_week_start = last_week_end - datetime.timedelta(days=6)
res['nixtlar'] = {
    'total': r_package_downloads(None, this_week_end),
    'this_week': r_package_downloads(this_week_start, this_week_end),
    'last_week': r_package_downloads(last_week_start, last_week_end),
}

# gen markdown
df = pd.DataFrame(res).T.reset_index().rename(columns={'index': 'library'})
total_downloads_this_week = df['total'].sum()
total_downloads_last_week = total_downloads_this_week - df['this_week'].sum()
this_week_downloads = df['this_week'].sum()
last_week_downloads = df['last_week'].sum()
libs_total = '\n'.join(f"  * {fmt_with_change(lib, total, total - this_week)}" for _, (lib, total, this_week, _) in df.iterrows())
libs_weekly = '\n'.join(f"  * {fmt_with_change(lib, this_week, last_week)}" for _, (lib, _, this_week, last_week) in df.iterrows())
report = f"""# Growth Report

* Total downloads: {fmt_with_change('', total_downloads_this_week, total_downloads_last_week)}
{libs_total}

* Weekly downloads: {fmt_with_change('', this_week_downloads, last_week_downloads)}
{libs_weekly}
"""
with open(f'os_report_{this_week_start}-{this_week_end}.md', 'wt') as f:
    f.write(report)
